package com.example.union_ad_ssgf.page

import android.util.Log
import com.example.union_ad_ssgf.event.AdEventAction
import com.example.union_ad_ssgf.event.AdRewardEvent
import com.qq.e.ads.rewardvideo.RewardVideoAD
import com.qq.e.ads.rewardvideo.RewardVideoADListener
import com.qq.e.ads.rewardvideo.ServerSideVerificationOptions
import com.qq.e.comm.util.AdError
import io.flutter.plugin.common.MethodCall
import java.util.Locale

/**
 * 激励视频页面
 */
class RewardVideoPage() : BaseAdPage(), RewardVideoADListener {
    private val TAG = RewardVideoPage::class.java.getSimpleName()

    /**
     * 激励广告对象
     */
    private lateinit var rewardVideoAD: RewardVideoAD

    /**
     * 设置激励视频服务端验证的自定义信息
     */
    private lateinit var customData: String

    /**
     * 设置服务端验证的用户信息
     */
    private lateinit var userId: String

    override fun loadAd(call: MethodCall?) {
        /**
         * 获取对应参数
         */
        val playMuted: Boolean = call!!.argument("playMuted")!!
        customData = call.argument("customData")!!
        userId = call.argument("userId")!!

        // 1. 初始化激励视频广告
        rewardVideoAD = RewardVideoAD(activity, posId, this, !playMuted)

        // 2. 设置服务端验证信息
        val options = ServerSideVerificationOptions.Builder()
            .setCustomData(customData)
            .setUserId(userId) // 设置服务端验证的用户信息
            .build()

        // 3. 服务端验证信息
        rewardVideoAD.setServerSideVerificationOptions(options)

        // 4. 加载激励视频广告
        rewardVideoAD.loadAD()
    }

    /**
     * 广告加载成功，可在此回调后进行广告展示
     **/
    override fun onADLoad() {
        Log.i(TAG, "onADLoad");
        rewardVideoAD.showAD();
        sendEvent(AdEventAction.onAdLoaded);
    }

    /**
     * 视频素材缓存成功，可在此回调后进行广告展示
     */
    override fun onVideoCached() {
        Log.i(TAG, "onVideoCached")
    }

    /**
     * 激励视频广告页面展示
     */
    override fun onADShow() {
        Log.i(TAG, "onADShow")
        sendEvent(AdEventAction.onAdPresent)
    }

    /**
     * 激励视频广告曝光
     */
    override fun onADExpose() {
        Log.i(TAG, "onADExpose")
        sendEvent(AdEventAction.onAdExposure)
    }

    /**
     * 激励视频触发激励（观看视频大于一定时长或者视频播放完毕）
     * @param map 若选择了服务端验证，可以通过 ServerSideVerificationOptions#TRANS_ID 键从 map 中获取此次交易的 id；若未选择服务端验证，则不需关注 map 参数。
     */
    override fun onReward(map: MutableMap<String, Any>?) {
        val transId = map!![ServerSideVerificationOptions.TRANS_ID] as String?
        Log.i(TAG, "onReward $transId") // 获取服务端验证的唯一 ID
        sendEvent(AdRewardEvent(posId,"",1,AdEventAction.onAdReward, transId!!, customData, userId))
    }

    /**
     * 激励视频广告被点击
     */
    override fun onADClick() {
        Log.i(TAG, "onADClick");
        sendEvent(AdEventAction.onAdClicked);
    }

    /**
     * 激励视频广告被关闭
     */
    override fun onVideoComplete() {
        Log.i(TAG, "onVideoComplete");
    }

    /**
     * 激励视频广告被关闭
     */
    override fun onADClose() {
        Log.i(TAG, "onADClose");
        sendEvent(AdEventAction.onAdClosed);
    }

    /**
     * 广告流程出错
     */
    override fun onError(error: AdError) {
        val msg = java.lang.String.format(
            Locale.getDefault(),
            "onError, error code: %d, error msg: %s",
            error.errorCode, error.errorMsg,
        )
        Log.i(TAG, "onError, adError=$msg")
        sendErrorEvent(error.errorCode, error.errorMsg)
    }
}